/******************************************************************************

	jiho.c

	JIHO signal decording module

	(c) 2013/08/13 Suwa   http://www.suwa-koubou.jp

    2013/12/11  update to support analog input for SBDBT32

*****************************************************************************/

#include <stdio.h>
#include "HardwareProfile.h"
#include "rtc.h"
#include "schedule.h"


/*------------------------------------------------------------------------------
	definition
------------------------------------------------------------------------------*/

#define	JIHO        (get_jiho())
#define LVL_HIGH    496    // 0.05V/3.3V*32768=496


/*------------------------------------------------------------------------------
	function prototype
------------------------------------------------------------------------------*/

void  rtc_adjust_byJIHO(void);

static void  set_adcin(void);
static void  reset_adcin(void);
static unsigned int get_jiho(void);


/*------------------------------------------------------------------------------
	global variable
------------------------------------------------------------------------------*/


/*===============================================================================
	adjust time by NHK radio2 JIHO     detect 440, 880 signal
===============================================================================*/

void  rtc_adjust_byJIHO(void)
{
	char data[7], cdata[7];
	int  n, count, flg440;
	char buf[32];
	DWORD st;

	printf("\nNow Time is \"%s\"\n", get_rtcstr(buf));

	// saved now time
	get_rtc(data);

	// is now time 59m?
	if(data[4] != 59){
		printf("Now Time isn't JIHO detecting time.\r\n");
		return;
	}

	// copy time data.
	for(n = 0; n < 7; n++) cdata[n] = data[n];

	// just before recording time?
	// set min,sec=0
	cdata[4] = cdata[5] = 0; 
	// set ++hour
	cdata[3]++;
    if(cdata[3] > 23){
       cdata[3] = 0; // hour
       cdata[6]++;   // wday
	}
	if(check_recording_time(cdata)){
		printf("Skip time adjust, because now time is just before recording time.\r\n");
		return;
	}

	isTimeAdjusted = 0;

	// enable A/D convert
	set_adcin();

	AplTimer1 = 30000L;	// during 30sec  59m45sec -> 00m15sec

retry:
	// wait 900msec no signal
	flg440 = 0;
	while((st = AplTimer1) != 0){
		if(JIHO){
			while(JIHO && AplTimer1);
		}else{
			while((!JIHO) && AplTimer1);
		}
		if((AplTimer1 - st)>890) break;
	}

next_tone:
	if(!AplTimer1){
		printf("Doesn't detected JIHO signal!\r\n");
		reset_adcin(); // disable A/D convert
		return;
	}
	// check 440Hz or 880Hz
	AplTimer2 = 990;
	count = 0;
	while(AplTimer2 && AplTimer1){
		count++;
		while(JIHO && AplTimer1);
		while(!JIHO && AplTimer1);
	}

	if(flg440 && count >= 840 && count <= 920){
		// time adjust
		adjust_time(data);
		isTimeAdjusted = 1;
		pwr_led_on();
		printf("Detected JIHO signal. Time adjusted!\r\n");
		printf("Adjusted time is \"%s\"\n", get_rtcstr(buf));
		reset_adcin(); // disable A/D convert
		return;

	}else if(count >= 36 && count <= 52){
		flg440++;
		goto next_tone;
	}else{
		goto retry;
	}
}

/*------------------------------------------------------------------------------
	enable A/D convert
------------------------------------------------------------------------------*/
static
void set_adcin(void)
{
	AD1CON1bits.ADON = 0;     // turn OFF AD module
	AD1CON1bits.ADSIDL = 0;   // continue module operation in idle mode
	AD1CON1bits.FORM = 0b011; // Signed Fractional 16-bit (DOUT = 0000 0000 0000 0000 sddd dddd dd00 0000)
	//AD1CON1bits.FORM = 0b010; // (DOUT = 0000 0000 0000 0000 dddd dddd dd00 0000)
	AD1CON1bits.SSRC = 0b111; // internal counter end sampling and starts conversion.(auto-convert)
	AD1CON1bits.CLRASAM = 0;  // Normal operation
	AD1CON1bits.ASAM = 0;     // Sampling begins when SAMP bit is set
	AD1CON1bits.SAMP = 0;     // yet sampling
	AD1CON1bits.DONE = 0;     // yet conversion(this bit is status bit)

	AD1CON2bits.VCFG = 0b000; // Voltage reference select AVdd/AVss
	AD1CON2bits.OFFCAL = 0;   // Disable Offset Calibration mode
	AD1CON2bits.CSCNA = 1;    // MUX-A Scan inputs
	AD1CON2bits.SMPI = 0;     //Interrupts at the completion of conversion for each 16th. sample/convert sequence
	AD1CON2bits.BUFM = 0;     // Buffer configured as one 16-word buffer
	AD1CON2bits.ALTS = 0;     // Always uses MUX A input multiplexer settings

	AD1CON3bits.ADRC = 0;	  // Clock derived from Peripheral Bus Clock (PBCLK)
	// AD1CON3bits.SAMC = 22;	  // Auto sample time is 23 * Tad, Best test result is 22.
	// AD1CON3bits.ADCS = 12;    // AD conversion clock select, Tad = 26Tpb
	// Tpb=80MHz 0.0125usec
	// Tad=26*Tpb=26*0.0125=0.325usec
	// (23+12)*Tad=35*0.325=11.375usec
	// 11.375*2=22.75usec (43.956kHz 43956/44100=0.996735=-3.265%)
	AD1CON3bits.SAMC = 28;	  // Auto sample time is 28 * Tad.
	AD1CON3bits.ADCS = 19;    // AD conversion clock select, Tad = (19+1)*2*Tpb=40Tpb
	// Tpb=80MHz 0.0125usec
	// Tad=40*Tpb=40*0.0125=0.5usec
	// (28+12)*Tad=40*0.5=20usec

	AD1CHSbits.CH0NB = 0;	// Channel 0 positive input select for MUX B,Vr-
	AD1CHSbits.CH0SB = 0;	// Channel 0 positive input select for MUX B,AN0
	AD1CHSbits.CH0NA = 0;	// Channel 0 positive input select for MUX A,Vr-
	AD1CHSbits.CH0SA = 0;	// Channel 0 positive input select for MUX A,AN0

	// AD1CSSL = 0b0000000000001100;	// AD input pin scan select, AN2,AN3
	// AD1PCFG = 0b1111111111110011;	// AN2,AN3 is analog input

	AD1CSSL = 0b0000000000000001;	// AD input pin scan select, AN0
	AD1PCFG = 0b1111111111111110;	// AN0 is analog input

	IFS1bits.AD1IF = 0;	// clear interrupt flag
	// IEC1bits.AD1IE = 1;	// enable AD1 interrupt for conversion done 
	// IPC6bits.AD1IP = 3; // priority is level 3.

	// start the ADC
	AD1CON1bits.ON = 1;
}


/*------------------------------------------------------------------------------
	disable A/D convert
------------------------------------------------------------------------------*/
static
void reset_adcin(void)
{
	IEC1bits.AD1IE = 0;	// disable AD1 interrupt for conversion done 
	IFS1bits.AD1IF = 0;	// clear interrupt flag
	AD1PCFG = 0b1111111111111111;	// AN0 is digital I/O

	// stop the ADC
	AD1CON1bits.ON = 0;
}


/*------------------------------------------------------------------------------
	read jiho signal
------------------------------------------------------------------------------*/
static
unsigned int get_jiho(void)
{
	short int data;

	AD1CON1bits.SAMP = 1;
	while(!AD1CON1bits.DONE);  // yet conversion
	data = ADC1BUF0;
	AD1CON1bits.DONE = 0;
	AD1CON1bits.SAMP = 0;
	IFS1bits.AD1IF = 0;	// clear interrupt flag

	if(data > LVL_HIGH) return 1;
	return 0;
}


/*** end of jiho.c ************************************************************/






