/******************************************************************************

    main.c

	Radio Recorder with SBDBT32 and M6951


    between SBDBT32 and M6951 connection

    << SBDBT >>               << M6951 >>
    J4 1(1)   MCLR
       2(2)   +3V3in  ------------ VCC  ------ AC Adapter(DC+3V3)   
       3(3)   GND     ------------ GND/AGND
       4(4)   PGED(AN0/RB0) ------ LOUT 
       5(5)   PGEC(AN1/RB1) ------ ROUT 

    J3 1(6)   RTS(RD1) ----------- U4TX
       2(7)   TX (RD3/I2C3-CLK) -- CLK
       3(8)   RX (RD2/I2C3-DAT) -- DAT
       4(9)   CTS(RD9) ----------- U4RX
       5(10)  STO(RB9) ----------- PON



	NHK 2nd. educaion 83.600MHz  Tuned(83600);
	FM Osaka Stereo   85.100MHz  Tuned(85100);
        FM COCOLO Stereo  76.500MHz  Tuned(76500);
	FM 802 Stereo     80.200MHz  Tuned(80200);

	(c) 2013/11/19 Suwa   http://www.suwa-koubou.jp
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "pic_delay.h"
#include "rtc.h"
#include "akc6951.h"
#include "record.h"
#include "schedule.h"
#include "jiho.h"
#include "uart2.h"

/*------------------------------------------------------------------------------
	Configuration for PIC32MX695F
------------------------------------------------------------------------------*/

#include "p32mx695f_config.h"


/*------------------------------------------------------------------------------
	definition
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
	function prototype
------------------------------------------------------------------------------*/

void    tuned_test(void);
void    InitializeSystem(void);
void    do_cmd(unsigned char cmd);
void	getstr(char *buf);


/*------------------------------------------------------------------------------
	variable
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
	
------------------------------------------------------------------------------*/

void test(void)
{
	printf("NHK radio 2, 2minutes recording.\n");
	recording(83600L, "08132100.WAV", 120*1000L);
}


/*==============================================================================
	main
===============================================================================*/

int main ( void )
{
	unsigned char cmd;


    // I/O initialize 
    InitializeSystem();

	// rtc initialize
	rtc_init();

    // UART2 initialize
	UART2Init();

	// DSP Radio initialize
	akc6951_initialize();


	// Stream stdout, stderr set unbuffered
	setbuf(stdout, NULL);
	setbuf(stderr, NULL);

	// display program title.
	printf("\n*** Radio Program Auto Recording System.\n");

	// load and dump recording reservation.
	//printf("Load and Dump recording information.\n");
	load_schedule();
	dump_schedule();


	// process loop
	for(;;){

		// put menu
		printf("\n*** Radio Recorder operation menu *****\n");
		printf("d: display now time.\n");
		printf("s: set now time.\n");
		printf("t: tuned test.\n");
		printf("r: recording test.\n");
		printf("l: list recording schedule.\n");
		printf("=>");

		// wait for command input
		while(!UART2IsPressed()){

			// rtc adjusted time?
			if(isRtcAdjustTime){
				// set tune NHK radio 2
				Tuned(83600L);
				Tuned(83600L);
				rtc_adjust_byJIHO();
				isRtcAdjustTime = 0;
			}

			// changed minutes?
			if(UpdateMinutes){
				UpdateMinutes = 0;
				schedule_recording();
			}

		}

		cmd = UART2GetChar();
		printf("%c\n", cmd);

		// do command
		do_cmd(cmd);

	}

}


/*===============================================================================
	do command 
===============================================================================*/

void do_cmd(unsigned char cmd)
{
	char  buf[128];
	int	  n, i;

	// do command
	switch(cmd){
	case 'd': // display now time.
		printf("%s\n", get_rtcstr(buf));
		break;

	case 's': // set now time.
		printf("input time ie. \"2013/07/16(2) 12:34:56\" ==>");
		getstr(buf);
		set_rtcstr(buf);
		break;

	case 't':
		tuned_test();
		break;

	case 'r':
		test();
		break;
		
	case 'l':
		load_schedule();
		printf("Record information list.\n");
		printf("-----------------------------------------------------------------\n");
		n = get_num_record_info();
		for(i = 0; i < n; i++){
			get_record_info(buf, i);
			printf("%s\n", buf);
		}
		printf("-----------------------------------------------------------------\n");
		break;

	default:
		break;
	} // switch

}


/*---------------------------------------------------------------------------
	tuned test
---------------------------------------------------------------------------*/

void tuned_test(void)
{
	int	 c, res;
	char buf[32];
	int	 val, boost, speaker;
	long freq;

	boost = 0;
	speaker = 1; 

	// do Radio control test
	for(;;){

		// put message of test start
		printf("\n*** DSP Radio Control Menu ***\n"); 
		printf("d:AKC6951 I2C Register dump.\n");
		printf("f:Frequency tunning.\n");
		printf("v:Volume setting.\n");
		printf("b:toggle Base Boost.\n");
		printf("s:toggle dual/single speaker.\n");
		printf("r:return to system menu.\n");
		printf("=>");

		while(!UART2IsPressed());
		c = UART2GetChar();
		printf("%c\n", c);

		switch(c){
		case 'b':
			// Base Boost ON/OFF
			boost ^= 1;
			akc6951_BaseBoost(boost);
			break;

		case 'd':
			// register dump
			printf("*** Register Dump ******\n");
			akc6951_register_dump();
			printf("************************\n");
			break;

		case 'f':
			// frequency tunning
			res = -1;
			printf("Frequency =>");
			getstr(buf);
			freq = atol(buf);
			if(freq >= 76000 && freq <=90000){
				res = akc6951_FM_tunning(freq);
			}else if(freq >= 522 && freq <= 1620){
				res = akc6951_AM_tunning(freq);
			}
			if(res == 0){
				printf("Tunning complete!!\n");
			}else{
				printf("Tuned Error.\n");
			}
			break;

		case 'v':
			// volume setting
			printf("Volume(0-63) =>");
			getstr(buf);
			val = atoi(buf);
			if(val < 0 || val > 63){
				printf("illegal volume value!\n");
			}else{
				akc6951_VOL_setting(val);
			}
			break;

		case 's':
			// select speaker
			speaker ^= 1;
			akc6951_SelectSpeaker(speaker);
			break;

		case 'r':
			return;

		default:
			printf("\n");
			break;
		} // switch

	} // for 
}

/*------------------------------------------------------------------------------
	getstr
------------------------------------------------------------------------------*/

void	getstr(char *buf)
{
	int	c;

	while(1){
		if(UART2IsPressed()){
			c = UART2GetChar();
			if(c >= ' ' && c <= 0x7e){
				*buf++ = c;
				UART2PutChar(c);       // echo back
			}else if(c == '\r'){
				UART2PrintString("\r\n");	// echo back
				break;
			}
		}
	}
	*buf = 0;

} // getstr


/*===============================================================================
	InitializeSystem()  SBDBT32 micom board
===============================================================================*/

void InitializeSystem ( void )
{
	int	value;

	// Set System Config
	//SYSTEMConfigPerformance(GetSystemClock());
	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

	// Enable the cache for the best performance	
	CheKseg0CacheOn();

	// enable multi vector
	INTEnableSystemMultiVectoredInt();

	value = OSCCON;
	while (!(value & 0x00000020)){
		value = OSCCON;    // Wait for PLL lock to stabilize	
	}

	INTEnableInterrupts();


	// ADC input off
	AD1PCFG = 0xffff;
	// disable JTAG
	DDPCONbits.JTAGEN = 0;

	//--- setting I/O pins               
    TRISB= 0x0023;
    LATB = 0xc200;
				// RB0(AN0/PGED)   LOUT    input 
				// RB1(AN1/PGEC)   ROUT    input
                // RB5             MMC_CD  input
				// RB9             PON     output, initial high
				// RB14            LED     output, LED monitor, initial high
                // RB15            LED     output, LED monitor, initial high
				// others                  output, output level low

	TRISC= 0x0000;
	LATC = 0x0000;
                // RC all pins not use

	TRISD= 0x0200;
	LATD = 0x000e;
                // RD1             U4TX      output  initial High
                // RD2(I2C3-DAT)   i2c data  input/output
                // RD3(I2C3-CLK)   i2c clock output
				// RD9             U4RX      input
                // others                    output  output level low

	TRISE= 0x0000;
	LATE = 0x0000;
                // RE all pins not use

	TRISF= 0x0000;
	LATF = 0x0000;
                // RF all pins not use

	TRISG= 0x0080;
	LATG = 0x0200;
                // RG6(SCK2)   output, MMC_SCK
                // RG7(SDI2)   input,  MMC_SDO
                // RG8(SDO2)   output, MMC_SDI
                // RG9(CS)     output, MMC_CS   initial level high

	//--- internal pullup
	CNPUEbits.CNPUE7=1;	// RB5(CN7) MMC_CD Pull-up
                        // other pins low level output, don't pull up.
	//--- setting I2C ---------------------------------------------
	I2C3CON = 0;	// disable I2C, set parameter default value.
	I2C3BRG = 98;	// PBCLK=80000KHz, FSCK=400KHz,
					// (PBCLK/(2*FSCK))-2=(80000/(2*400))-2=98
	I2C3ADD = 0x57;	// slave module own address, not use. 0x57 is same 0xAE.

} // InitializeSystem


/*** end of main.c *************************************************************/
