/*****************************************************************

	schedule.c

	(c) 2013/08/02 Suwa   http://www.suwa-koubou.jp

     2013/11/19 for SBDBT32
*****************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "rtc.h"
#include "record.h"
//#include "uart2.h"

/*----------------------------------------------------------------
	definition
----------------------------------------------------------------*/

struct schedule_t{
	unsigned long frequency;
	char year;	// -1(every)
	char month;	// -1(every)
	char day;	// -1(every)
	char wday;	// -1(don't care), 0-6(sun-sat)
	char bhh;
	char bmm;
	char ehh;
	char emm;
};

#define MAX_RESERVE   40 	// 7day * 5reserve/day + 5


/*----------------------------------------------------------------
	function prototype
----------------------------------------------------------------*/

void schedule_recording(void);
void load_schedule(void);
void dump_schedule(void);
static int add_schedule(struct schedule_t *sch);
struct schedule_t *check_recording_time(char td[]);


/*----------------------------------------------------------------
	global variable
----------------------------------------------------------------*/

static char *wdaynm[] = { "sun", "mon", "tue", "wed", "thu", "fri", "sat", };
static struct schedule_t *reserve[MAX_RESERVE];
static int numreserve;


/*================================================================
	do schedule
================================================================*/

void schedule_recording(void)
{
	struct schedule_t *p;
	char td[7];
	char fname[16];
	int  rhh, rmm;
	unsigned long rectime;


	printf("enter schedule_recording()\n");


	// get now time
	get_rtc(td);

	// is now time recording begin time
	printf("Check recoding time?\r\n");
	p = check_recording_time(td);
	if(!p){
		printf("Now isn't recording time.\r\n");
		return;
	}

	// make file name as "MMDDhhmm.WAV"
	sprintf(fname, "%02d%02d%02d%02d.WAV", td[1], td[2], td[3], td[4]);

	// calculate recording times
	rhh = (int)p->ehh - (int)p->bhh;
	if(rhh < 0) rhh += 24;
	rmm = (int)p->emm - (int)p->bmm;
	if(rmm < 0){
		rmm += 60;
		rhh--;
	}
	rectime = ((long)rhh * 3600 + (long)rmm * 60) * 1000L;
	
	// do recording
	printf("Now recording start. Freq=%ld,fname=%s,rectime=%ldsec\n", p->frequency, fname, rectime/1000);
	recording(p->frequency, fname, rectime);
	printf("Recording complete.\n");

}

/*----------------------------------------------------------------
	check and get recording info
----------------------------------------------------------------*/

struct schedule_t *check_recording_time(char td[])
{
	struct schedule_t *p;
	int	n;

	for(n = 0; n < numreserve; n++){
		p = reserve[n];

		if(p->year != -1){
			if(p->year != td[0]) continue;
		}
		if(p->month != -1){
			if(p->month != td[1]) continue;
		}
		if(p->day != -1){
			if(p->day != td[2]) continue;
		}
		if(p->wday != -1){
			if(p->wday != td[6]) continue;
		}

		if(p->bhh == td[3] && p->bmm == td[4]){
			return p;
		}
	}
	return NULL;

}



/*----------------------------------------------------------------
	load schedule data from disk file.
	Disk file name is "RECORD.TXT"

	#*** schedule plan order form
	# 
	# format1  order one day
	#   freq yyyy/mm/dd hh:mm hh:mm
	# format2  every week day
	#   freq wday hh:mm hh:mm
	#   freq wday-wday hh:mm hh:mm

	86500 2013/12/01 05:00 06:00
	83600 mon-fri 06:45 07:00
	76500 mon-fri 08:30 08:40
	85100 sat     07:00 07:50
	80200 sun     16:30 17:45
	78500 sat-mon 01:20 01:35
----------------------------------------------------------------*/

void load_schedule(void)
{
	struct schedule_t sch;
	FILE *fin;
	int	n, bwday, ewday;
	char buf[128], *p;

	fin = fopen("RECORD.TXT", "r");
	if(fin == NULL){
		printf("RECORD.TXT open error.\n");
		return;
	}

	// clear now reserved list
	for(n = 0; n < numreserve; n++){
		if(reserve[n]){
			free(reserve[n]);
		}
	}
	numreserve = 0;

	while(fgets(buf, 128, fin) != NULL){
		p = buf;
		if(*p == '\n' || *p == '\r' || *p == '#') continue;
		// frequency ...
		while(*p == ' ')p++;
		sch.frequency = atol(p); while(isdigit(*p)) p++;
		if(*p != ' ') continue;
		while(*p == ' ') p++;

		// 20yy/mm/dd bhh:bmm ehh:emm
		if(*p == '2'){	
			sch.year = atoi(p)-2000; while(isdigit(*p)) p++;
			if(*p++ != '/') continue;	// error
			sch.month = atoi(p); while(isdigit(*p)) p++;
			if(*p++ != '/') continue;	// error
			sch.day = atoi(p); while(isdigit(*p)) p++;
			if(*p != ' ') continue;
			while(*p == ' ') p++;
			sch.bhh = atoi(p); while(isdigit(*p)) p++;
			if(*p++ != ':') continue;	// error
			sch.bmm = atoi(p); while(isdigit(*p)) p++;
			if(*p != ' ') continue;
			while(*p == ' ') p++;
			sch.ehh = atoi(p); while(isdigit(*p)) p++;
			if(*p++ != ':') continue;	// error
			sch.emm = atoi(p);
			sch.wday = -1;
			add_schedule(&sch);
			continue;
		}
		// wday-wday bhh:bmm ehh:emm
		// wday bhh:bmm ehh:emm
		for(n = 0; n < sizeof(wdaynm)/sizeof(wdaynm[0]); n++){
			if(strncmp(p, wdaynm[n], 3) == 0) break;
		}
		if(n >= sizeof(wdaynm)/sizeof(wdaynm[0])) continue;
		bwday = ewday = n;
		p+=3; while(*p == ' ') p++;
		if(*p == '-'){
			p++; while(*p == ' ') p++;
			for(n = 0; n < sizeof(wdaynm)/sizeof(wdaynm[0]); n++){
				if(strncmp(p, wdaynm[n], 3) == 0) break;
			}
			if(n >= sizeof(wdaynm)/sizeof(wdaynm[0])) continue;
			ewday = n;
			p+=3; while(*p == ' ') p++;
		}

		sch.bhh = atoi(p); while(isdigit(*p)) p++;
		if(*p++ != ':') continue;	// error
		sch.bmm = atoi(p); while(isdigit(*p)) p++;
		if(*p != ' ') continue;
		while(*p == ' ') p++;
		sch.ehh = atoi(p); while(isdigit(*p)) p++;
		if(*p++ != ':') continue;	// error
		sch.emm = atoi(p);

		sch.year = sch.month = sch.day = -1;

		if(bwday > ewday) ewday += 7;
		for(n = bwday; n <= ewday; n++){
			sch.wday = n;
			if(sch.wday > 6) sch.wday -= 7;			
			add_schedule(&sch);
		}
	}

	fclose(fin);

}

/*----------------------------------------------------------------
	add a schedule into recording reservation list
----------------------------------------------------------------*/
static
int add_schedule(struct schedule_t *sch)
{
	struct schedule_t *p;

	// list over?
	if(numreserve >= MAX_RESERVE){
		printf("too many reservation of recording!\n");
		return -1;
	}

	// check data
	if( ( sch->frequency < 75000 || sch->frequency > 91000 ) || 
		((sch->year != -1 && sch->year < 1) || sch->year > 99)  ||
		((sch->month != -1 && sch->month < 1) || sch->month > 12)  ||
		((sch->day != -1 && sch->day < 1) || sch->day > 31) ||
		((sch->wday != -1 && sch->wday < 0) || sch->wday > 6) ||
		( sch->bhh < 0 || sch->bhh > 23) ||
		( sch->bmm < 0 || sch->bmm > 59) ||
		( sch->ehh < 0 || sch->ehh > 23) ||
		( sch->emm < 0 || sch->emm > 59) 	){
		printf("parameter error!\n");
		return -1;
	}

	// allocate info item
	p = (struct schedule_t *)malloc(sizeof(struct schedule_t));
	if(p == NULL){
		printf("not enough memory!\n");
		return -1;
	}

	// copy reservation data
	p->frequency = sch->frequency;
	p->year = sch->year;
	p->month = sch->month;
	p->day = sch->day;
	p->wday = sch->wday;
	p->bhh = sch->bhh;
	p->bmm = sch->bmm;
	p->ehh = sch->ehh;
	p->emm = sch->emm;

	// add to list
	reserve[numreserve++] = p;
	return 0;
}


/*----------------------------------------------------------------
	dump schedule reservation list
----------------------------------------------------------------*/

void dump_schedule(void)
{
	struct schedule_t *p;
	int	n;

	printf("Dump recording information.\n");
	if(numreserve == 0){
		printf("nothing reserve info.\n");
		return;
	}
	for(n = 0; n < numreserve; n++){
		p = reserve[n];
		printf("Freq=%ld,year=%02d,month=%02d,day=%02d,wday=%d  begin_hh:mm=%02d:%02d end_hh:mm=%02d:%02d\n",
			p->frequency, p->year, p->month, p->day, p->wday, p->bhh, p->bmm, p->ehh, p->emm);
	}

}


/*----------------------------------------------------------------
	get number of record info
----------------------------------------------------------------*/

int	get_num_record_info(void)
{
	return numreserve;
}

/*----------------------------------------------------------------
	get record info
----------------------------------------------------------------*/

void get_record_info(char *buf, int num)
{
	struct schedule_t *p;

	p = reserve[num];
	sprintf(buf, "Freq=%ld,year=%02d,month=%02d,day=%02d,wday=%d  begin_hh:mm=%02d:%02d end_hh:mm=%02d:%02d",
			p->frequency, p->year, p->month, p->day, p->wday, p->bhh, p->bmm, p->ehh, p->emm);
}

/*** end of schedule.c ******************************************/

